<?php

//builds the form for configuring transactions.
function default_currency_config(){
  $saved = variable_get('currency0', array());
  if (!count($saved))drupal_set_message('no currency defined in variables table');
  if ($saved['symbol'])$h = '('.$saved['symbol'].')';
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol or name of of currency ').$h,
    '#default_value' => $saved['symbol'],
    '#size' => 6,
    '#maxlength' => 9,
    '#description' => t("You can use any text or html entity which is likely to be available to your users' browsers. Put it's html value here, e.g. '&amp;#x01B1;' renders as '&#x01B1;' Otherwise just use the plural name of the currency unit and select after, below.")
  );
///sites/bazaar.localhost/files/omega16.png
  $form['icon'] = array(
    '#type' => 'textfield',
    '#title' => 'Path to an icon file',
    '#default_value' => $saved['icon'],
    '#size' => 60,
    '#maxlength' => 132,
    '#description' => t("Instead of using a symbol, you can upload a graphic. Ideally this graphic should be a similar size to the text, i.e. around 12x12 pixels. The path can be anywhere on the internet, in which case it should begin with 'http://', or on your domain, in which case it should begin with '/'. For an example, use this: <b>sites/bazaar.localhost/files/omega16.png</b>")
  );
  $form['after'] = array(
    '#type'=>'radios',
    '#title' => t('Where does the symbol go in relation to the amount?'),
	'#return_value'=>1,
    '#default_value' => $saved['after'],
	'#options' => array(0=>t('before'), 1=>t('after')),
    '#description' => t("Many schemes just write their unit of currency after the figure, as it is spoken, e.g. '6 units', but more established currencies use a symbol before the amount in writing, e.g. '$6'.")
  );
  $form['max_balance'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum Balance'),
    '#default_value' => $saved['max_balance'],
    '#size' => 4,
    '#maxlength' => 6,
    '#description' => t("Transactions will be declined by the system if the balance of the recipient would go above this number. Up to six digits. Leave blank for no limit. This feature works independently of the maximum balance for each additional currency defined by the currencies module.")
);
  $form['min_balance'] = array(
    '#type' => 'textfield',
    '#title' => t('Minimum Balance'),
    '#default_value' => $saved['min_balance'],
    '#size' => 4,
    '#maxlength' => 6,
    '#description' => t("Transactions will be declined by the system if the balance of the payer would go below this number. Up to seven digits, including minus sign. Leave blank for no limit (not reccomended). This feature works independently of the minimum balance for each additional currency defined by the currencies module.")
);
  $form['cid'] = array(
    '#type' => 'hidden',
	'#value'=>0);

  $form['actions']['submit'] = array(
	'#type' => 'submit', 
	'#value' => t('Submit')
  );
  $form['#redirect'] = url('admin/settings');
  return $form;
}

function default_currency_config_validate($node, $form){
  if ($form['values']['max_balance'] < 0 || $form['values']['max_balance'] < $form['values']['min_balance'])form_set_error('default_currency_config', t('Maximum balance must be an integer larger than zero and minimum balance'));
  if ($form['values']['min_balance'] > 0 )form_set_error('default_currency_config', t('Maximum balance must not be larger than 0'));
  global $user; if ($user->uid != 1)
  form_set_error('default_currency_config', 'You are not permitted change these settings');
}

function default_currency_config_submit($form, &$form_state){
  foreach(array('op', 'submit', 'form_build_id', 'form_token', 'form_id') as $field){
    unset($form_state['values'][$field]);
  }
  variable_set('currency0', $form_state['values']);
}

function transaction_quality_config(){
  $already=variable_get('transaction_qualities', array());
  $default = implode("\n", $already);
  $form['qualities']= array(
    '#type'=>'textarea',
    '#default_value'=>$default,
	'#title'=>'Possible ratings',
	'#description' => t('Traders can earn a reputation when their customers rate them highly. Use this field to list the words used to describe the quality of transactions, from worst to best, one on each line. Example:<br />Bad<br />As expected<br />Great<br>
	   Note that changing the number of rating options after transactions have been entered will distort the averages. I wonder if there is a drupal ratings module which could do this more effectively? Probably not because only buyers can rate. This feature therefore needs a lot more work to make it attractive and interesting.')
  );
  $form['actions']['submit'] = array(
	'#type' => 'submit', 
    '#value' => t('Submit')
  );
  $form['#redirect'] = url('admin/settings');
  return $form;
}
function transaction_quality_config_submit($form, $form_state){
  $qualities = explode("\n", $form_state['values']['qualities']);
  foreach ($qualities as $index=>$rating){
    $qualities[$index] = trim($rating);
  }
  variable_set('transaction_qualities', $qualities);
}

//this form creates the transaction on the basis of data from a previous form (picked up from POST). It presents itself as a confirmation form but is in fact in implementation of Drupal's node/add hook_form

function checking_limits($transaction){
  //the transaction obj stores user objects in activeuser and otherperson, and ids in from_uid and to_uid
  //so we need to convert between them if we want to send the userobjects to the friendly names function without loading users again
  //get the buyer won't go below the minium
  if ($transaction->transaction_type==1 || $transaction->transaction_type==3){
    $from_user=$transaction->otherperson;
  }else{
    $from_user = $transaction -> activeuser;
  }
  $cid = $transaction->currency->cid;
  $pending = unserialize ($from_user->profile_currency_pending);
  if ($pending[$cid] - $transaction->quantity < $transaction->currency->min_balance){
    drupal_set_message(t('Transaction has been rejected as it would take !username below the minimum balance of !quantity', 
      array('!username' => user_friendly_name($from_user),
	    '!quantity' => theme_money($transaction->currency->min_balance, $cid)
	  )
	), 'warning');
	$reject = true;
  }
  
  if ($transaction->transaction_type==2 || $transaction->transaction_type==4){
    $to_user=$transaction->otherperson;
  }else{
    $to_user = $transaction -> activeuser;
  }
  $pending = unserialize ($to_user->profile_currency_pending);
  if ($pending[$cid] + $transaction->quantity > $transaction->currency->max_balance){
    drupal_set_message(t('Transaction has been rejected as it would take !username above the maximum balance of !quantity', 
      array('!username' => user_friendly_name($to_user),
	    '!quantity' => theme_money($transaction -> currency->max_balance, $cid)
	  )
	), 'warning');
	$reject = true;
  }
  return $reject;
}
//for each user id passed, go through all their transactions and calculate balance, turnover and pending amounts in their profile
function recalculate_balances($ids=array(), $cid=0){
  while ($uid = array_pop($ids)){
    //get all the transactions the user was involved in from the given currency
    $query = "SELECT DISTINCT t.nid, t.from_uid, t.to_uid, t.quantity, n.status
      FROM {transactions} t LEFT JOIN {node} n ON t.nid = n.nid 
      WHERE (t.from_uid = '%d' OR t.to_uid = '%d') AND t.cid = '%d'";
    $results=db_query($query, $uid, $uid, $cid);
    //add  up the total income, outgoing and incomplete transactions
    $turnover=0;
    $outgoings=0;
    $pending=0;
    while ($t = db_fetch_array($results)){
      if ($t['status']){ //completed transactions
        if ($t['from_uid']!= $uid){
          $turnover +=$t['quantity'];
        } else {
          $outgoings +=$t['quantity'];
        }
      } else { //unpublished transactions are pending
        if ($t['from_uid']!= $uid){
          $pending += $t['quantity'];
        } else {
          $pending -= $t['quantity'];
        }
      }
    }
    //getting the current balances out of the user profile, noting the fids for the next query
	//we have to get the balances out because they are stored as arrays of which we want to change one value in each
    $results=db_query("SELECT name, value, v.fid 
      FROM {profile_values} v LEFT JOIN {profile_fields} f ON v.fid = f.fid 
      WHERE f.category = 'currencies' AND uid = %d", $uid);
    
    $fields=array();
    //modifying the arrays to reflect the new amounts
    while ($field = db_fetch_array($results)){
      $fields[$field['name']]=unserialize($field['value']);
      unset($fields[$field['name']]['grain']);
      unset($fields[$field['name']]['']);
	}
    $fields['profile_currency_turnover'][$cid]=$turnover;
    $bal = $turnover - $outgoings;
    $fields['profile_currency_balance'][$cid]=$bal;
    $fields['profile_currency_pending'][$cid]=$bal + $pending;
    //putting the arrays back into the user profile values
    foreach ($fields as $fieldname=>$newvalue){
      db_query("UPDATE {profile_values} AS v JOIN {profile_fields} AS f ON v.fid = f.fid 
        SET v.value = '%s' 
        WHERE f.name = '%s' and v.uid = '%d'", 
        serialize($newvalue), $fieldname, $uid
      );
    }
  }
}

function _complete_transaction($nid){
  $transaction = node_load(array('nid'=>$nid));
  global $user;
   //rather than check in the menu all the time, it's more efficent to check access permission here
  //if the current user is either the otheruser, or has permission 
  if ($user->uid == $transaction->otherperson->uid || user_access(t('edit all transactions'))){
    db_query("UPDATE node SET status = 1 WHERE nid = %d", $nid);
    drupal_set_message(t("transaction complete!"));
  } else {
    drupal_set_message(t('you are not permitted to complete this transaction.'));
  }
  //recalculate the cached balances 
  recalculate_balances(array($transaction -> from_uid, $transaction -> to_uid), $transaction ->currency ->cid);
  drupal_goto('user');
}



?>