<?php
/**
 * @file
 * Contains default views on behalf of the transactions  module.
 */

 /**
 * Implementation of hook_views_data()
 */
function transactions_views_data() {
  $data = array();
  $data['transactions']['table']['group']  = t('Transactions');
  $data['transactions']['table']['join'] = array(
    'node' => array(
      'left_field' => 'nid',
      'field' => 'nid',
    ),
  );
	
  $data['transactions']['from_uid'] = array(
    'title' => t('Purchaser'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('The user id who spent currency in the transaction.'),
      'handler' => 'views_handler_field_transactions_from_id'
    ),
    'filter' => array(
      'help' => 'Show only transactions where the current user paid',
      'handler' => 'views_handler_filter_trader_current'
    ),
    'argument' => array(
      'help' => t('The user who spent currency in the transaction.'),
      'handler' => 'views_handler_argument'
    ),
    'relationship' => array(
      'help' => t('The user who spent currency in the transaction.'),
      'base' => 'users',
      'field' => 'from_uid',
      'handler' => 'views_handler_relationship',
      'label' => t('Purchaser')
    ),
  );
  $data['transactions']['to_uid'] = array(
    'title' => t('Recipient'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('The user id who received currency in the transaction.'),
      'handler' => 'views_handler_field_transactions_to_id',
    ),
    'filter' => array(
      'help' => 'Show only transactions where the current user was paid',
      'handler' => 'views_handler_filter_trader_current'
    ),
    'argument' => array(
      'help' => t('The user who received currency in the transaction.'),
      'handler' => 'views_handler_argument'
    ),
    'relationship' => array(
      'help' => t('The user who received currency in the transaction.'),
      'base' => 'users',
      'field' => 'to_uid',
      'handler' => 'views_handler_relationship',
      'label' => t('Recipient')
    ),
  );
  $data['transactions']['quality'] = array(
    'title' => t('Rating'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('Quality of the transaction according to the payer'),
      'handler' => 'views_handler_field'
    ),
  );
  $data['transactions']['quantity'] = array(
    'title' => t('Amount'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('Number of units of currency transferred'),
      'handler' => 'views_handler_field'
    ),
  );
  $data['transactions']['cid'] = array(
    'title' => t('Currency'), // The item it appears as on the UI,
    'field' => array(
      'help' => t('The currency that the transaction was conducted in'),
      'handler' => 'views_handler_field'
    ),
  );
  /*
  $data['node']['status'] = array(
    'title' => t('Completed'), // The item it appears as on the UI,
    'argument' => array(
      'help' => t('Whether the transaction is complete'),
      'handler' => 'views_handler_argument'
    ),
  );*/
  return $data;
}

function transaction_views_data_alter(&$data){
  //why isn't this hook firing?
  $data['node']['status']['argument'] = array(
    'handler'=>'views_handler_argument'
  );

} 
 
 
/**
 * Implementation of hook views_default_views().
 */
function transactions_views_default_views() {$view = new view;
$view->name = 'transactions_by_user';
$view->description = 'show transaction from or to a user';
$view->tag = 'transactions';
$view->view_php = '';
$view->base_table = 'node';
$view->is_cacheable = '0';
$view->api_version = 2;
$view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
$handler = $view->new_display('default', 'Defaults', 'default');
$handler->override_option('relationships', array(
  'from_uid' => array(
    'label' => 'Purchaser',
    'required' => 1,
    'id' => 'from_uid',
    'table' => 'transactions',
    'field' => 'from_uid',
    'relationship' => 'none',
  ),
  'to_uid' => array(
    'id' => 'to_uid',
    'table' => 'transactions',
    'field' => 'to_uid',
  ),
));
$handler->override_option('fields', array(
  'name_1' => array(
    'label' => 'Purchaser',
    'link_to_user' => 1,
    'exclude' => 0,
    'id' => 'name_1',
    'table' => 'users',
    'field' => 'name',
    'relationship' => 'from_uid',
  ),
  'title' => array(
    'label' => 'Trade',
    'link_to_node' => 0,
    'exclude' => 0,
    'id' => 'title',
    'table' => 'node',
    'field' => 'title',
    'relationship' => 'none',
  ),
  'quantity' => array(
    'label' => 'Amount',
    'exclude' => 0,
    'id' => 'quantity',
    'table' => 'transactions',
    'field' => 'quantity',
    'relationship' => 'none',
  ),
  'name_2' => array(
    'label' => 'Recipient',
    'link_to_user' => 1,
    'exclude' => 0,
    'id' => 'name_2',
    'table' => 'users',
    'field' => 'name',
    'relationship' => 'to_uid',
  ),
  'cid' => array(
    'label' => 'Currency',
    'exclude' => 1,
    'id' => 'cid',
    'table' => 'transactions',
    'field' => 'cid',
    'relationship' => 'none',
  ),
));
$handler->override_option('sorts', array(
  'created' => array(
    'order' => 'DESC',
    'granularity' => 'minute',
    'id' => 'created',
    'table' => 'node',
    'field' => 'created',
    'relationship' => 'none',
  ),
));
$handler->override_option('arguments', array(
  'from_uid' => array(
    'default_action' => 'empty',
    'style_plugin' => 'default_summary',
    'style_options' => array(),
    'wildcard' => 'all',
    'wildcard_substitution' => 'All',
    'title' => '',
    'default_argument_type' => 'fixed',
    'default_argument' => '',
    'validate_type' => 'none',
    'validate_fail' => 'not found',
    'id' => 'from_uid',
    'table' => 'transactions',
    'field' => 'from_uid',
    'relationship' => 'none',
    'default_argument_user' => 0,
    'default_argument_fixed' => '',
    'default_argument_php' => '',
    'validate_argument_node_type' => array(
      'transaction' => 0,
      'offer' => 0,
      'want' => 0,
      'page' => 0,
      'story' => 0,
    ),
    'validate_argument_node_access' => 0,
    'validate_argument_nid_type' => 'nid',
    'validate_argument_vocabulary' => array(
      '1' => 0,
    ),
    'validate_argument_type' => 'tid',
    'validate_argument_php' => '',
  ),
  'to_uid' => array(
    'default_action' => 'ignore',
    'style_plugin' => 'default_summary',
    'style_options' => array(),
    'wildcard' => 'all',
    'wildcard_substitution' => 'All',
    'title' => '',
    'default_argument_type' => 'fixed',
    'default_argument' => '',
    'validate_type' => 'none',
    'validate_fail' => 'not found',
    'id' => 'to_uid',
    'table' => 'transactions',
    'field' => 'to_uid',
    'relationship' => 'none',
    'default_argument_user' => 0,
    'default_argument_fixed' => '',
    'default_argument_php' => '',
    'validate_argument_node_type' => array(
      'transaction' => 0,
      'offer' => 0,
      'want' => 0,
      'page' => 0,
      'story' => 0,
    ),
    'validate_argument_node_access' => 0,
    'validate_argument_nid_type' => 'nid',
    'validate_argument_vocabulary' => array(
      '1' => 0,
    ),
    'validate_argument_type' => 'tid',
    'validate_argument_php' => '',
  ),
));
$handler->override_option('filters', array(
  'type' => array(
    'operator' => 'in',
    'value' => array(
      'transaction' => 'transaction',
    ),
    'group' => '0',
    'exposed' => FALSE,
    'expose' => array(
      'operator' => FALSE,
      'label' => '',
    ),
    'id' => 'type',
    'table' => 'node',
    'field' => 'type',
    'relationship' => 'none',
  ),
  'status' => array(
    'operator' => '=',
    'value' => 1,
    'group' => '0',
    'exposed' => FALSE,
    'expose' => array(
      'operator' => FALSE,
      'label' => '',
    ),
    'id' => 'status',
    'table' => 'node',
    'field' => 'status',
    'relationship' => 'none',
  ),
));
$handler->override_option('access', array(
  'type' => 'role',
  'role' => array(
    '2' => 2,
  ),
  'perm' => 'view advanced help topic',
));
$handler->override_option('title', 'Recent Purchases');
$handler->override_option('footer_format', '1');
$handler->override_option('empty', 'You haven\'t completed any transactions yet');
$handler->override_option('empty_format', '1');
$handler->override_option('items_per_page', 5);
$handler->override_option('use_pager', 'mini');
$handler->override_option('use_more', 1);
$handler->override_option('distinct', 1);
$handler->override_option('style_plugin', 'table');
  $views[$view->name] = $view;
  
  return $views;
}


class views_handler_filter_trader_current extends views_handler_filter_boolean_operator{
  function construct() {
    parent::construct();
    $this->value_value = t('Is the logged in user');
  }

  function query() {
    $this->ensure_my_table();
    $this->query->add_where($this->options['group'], "$this->table_alias.$this->real_field " . (empty($this->value) ? '!=' : '=') . " ***CURRENT_USER***");
  }
}


/**
 * Field handler to allow linking to a purchaser's name
 *
 * @ingroup views_field_handlers
 */
class views_handler_field_transactions_from_id extends views_handler_field {
  /**
   * Override init function to add uid
   */
  function init(&$view, &$data) {
    parent::init($view, $data);
  }

  function options(&$options) {
    parent::options($options);
    $options['link_to_purchaser'] = TRUE;
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $form['link_to_purchaser'] = array(
      '#title' => t("Link this field to its user, the purchaser's profile"),
      '#type' => 'checkbox',
      '#default_value' => $this->options['link_to_purchaser'],
    );
  }

  function render_link($data, $values) {
    $account->uid = $values->from_uid;
    $account->name = $values->name;
    ajax_dump($account);
    if (!empty($this->options['link_to_purchaser'])) {
      return theme('username', $account);
    }
    else {
      return $data;
    }
  }

  function render($values) {
    return $this->render_link(check_plain($values->{$this->field_alias}), $values);
  }
}

/**
 * Field handler to allow linking to a recipient's name
 *
 * @ingroup views_field_handlers
 */
class views_handler_field_transactions_to_id extends views_handler_field {
  /**
   * Override init function to add uid
   */
  function init(&$view, &$data) {
    parent::init($view, $data);
  }

  function options(&$options) {
    parent::options($options);
    $options['link_to_recipient'] = TRUE;
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $form['link_to_recipient'] = array(
      '#title' => t("Link this field to its user, the recipient's profile"),
      '#type' => 'checkbox',
      '#default_value' => $this->options['link_to_recipient'],
    );
  }

  function render_link($data, $values) {
    $account->uid = $values->to_uid;
    $account->name = $values->name; 
    if (!empty($this->options['link_to_recipient'])) {
      return theme('username', $account);
    } 
    else {
      return $data;
    }
  }

  function render($values) {
    return $this->render_link(check_plain($values->{$this->field_alias}), $values);
  }
}
/* VIEWS WE WILL NEED 
pending transactions of uid
Transactions waiting for uid to confirm
recent transactions with uid
recent incomings / outgoings of uid
transactions on the system, complete / incomplete / both
* Secondary ideas
Users with low/high quality ratings
Users with low/high turnover [recent]
Users with low/high balances
Users in categories */

function ajax_dump($my_variable){
  static $this_string;
  ob_start();
  var_dump($my_variable);
  $this_string.= "\n\n".ob_get_contents();
  ob_end_clean();
  
    $print_string = ereg_replace('>.', '>', $this_string);
  file_put_contents('ajax_report.txt',$print_string );
}
?>
